// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"
#include "scene/components/entity/EntityFaction.hpp"

namespace Dawn {
  class EntityAttackBase : public SceneItemComponent {
    protected:
      float_t attackTime = -1;

    public:
      StateProperty<bool_t> interrupted;
      StateEvent<> onAttackRampUpStart;
      StateEvent<> onAttackRampUpEnd;
      StateEvent<> onAttackActiveStart;
      StateEvent<> onAttackActiveEnd;
      StateEvent<> onAttackRampDownStart;
      StateEvent<> onAttackRampDownEnd;
      StateEvent<> onAttackCooldownStart;
      StateEvent<> onAttackCooldownEnd;
      StateEvent<> onAttackInterrupted;

      EntitySwordAttack(SceneItem* item);
      
      virtual float_t getAttackRampUpDuration() = 0;
      virtual float_t getAttackActiveDuration() = 0;
      virtual float_t getAttackRampDownDuration() = 0;
      virtual float_t getAttackCooldownDuration() = 0;
      virtual bool_t isInterruptable() = 0;

      virtual void attack() = 0;
      
      void interrupt();

      bool_t isAttacking();

      void onStart() override;
  };
}